
import numpy as np

from scipy import stats
from scipy import interpolate as intp
from astropy.table import Table

import matplotlib as mpl
#mpl.use('Agg')

from matplotlib import pyplot as plt

def apl(ax, n, af1, af2, cfn, lab='BNS', col='blue', sty='-', wid=3):
    alhlv  = Table.read(af1, format='ascii.fast_tab')
    alhlvk = Table.read(af2, format='ascii.fast_tab')
    coincs = Table.read(cfn, format='ascii.fast_tab')

    ifos  = coincs['ifos']
    ahlv  = alhlv ['area(90)']
    ahlvk = alhlvk['area(90)']

    if len(ahlv) != len(ahlvk):
        raise ValueError('Length is different: '+af1+' and '+af2)

    if len(ahlv) != len(ifos):
        raise ValueError('Length is different: '+af1+' and '+cfn)

    data = []
    for i in range(0, len(ahlv)):
        nd = 3 if ifos[i] == 'H1,L1,V1' else 2

        if n == nd:
            r = ahlvk[i]/ahlv[i]
            if r >= 1:
                r = 0.999
            data.append((1-r)*100)

    #t = np.linspace(0, 100, 100)
    t = np.geomspace(1, 100, 100)
    kde = stats.gaussian_kde(np.asarray(np.log(data)), bw_method=0.1)
    (std,), = np.sqrt(kde.covariance)
    y = stats.norm(kde.dataset.ravel(), std).\
        cdf(np.log(t)[:, np.newaxis]).mean(1)

    y /= y[-1]

    am = 0
    if y[0] < 0.5:
        ff  = intp.interp1d(y, t)
        am = ff(0.5)

    fmt = '4.1f' if am > 10 else '4.2f'

    if len(lab) == 3:
        lab += '  '

    lb = lab+' N='+str(n)+' ->'+str(n+1)+' median= '+format(am, fmt)+'%'

    ax.semilogx(t, y, color = col, linestyle = sty, linewidth = wid, \
                    label = lb)


def plot(mpc):
    fsiz = 14 
    fig  = plt.figure(dpi=80, facecolor='white')
    ax   = fig.add_subplot(1, 1, 1)

    ax.set_ylim(0, 1)
    ax.set_xticks([1, 10, 100])
    ax.set_yticks([0, 0.25, 0.50, 0.75, 1])
    ax.set_xlabel('Relative improvement in 90% credible area (%)',\
                                                   fontsize=fsiz)
    ax.set_ylabel('Cumulative fraction of events', fontsize=fsiz)

    plt.tick_params(labelsize = fsiz)

    ax.axhline(0.5, color='black', linestyle=':')

    col1 = '#0073b2'
    col2 = '#de8f07'
    col3 = '#029e73'
    srcs = [ 'bns', 'bbh', 'nsbh' ]
    cols = [  col1,  col2,   col3 ]

    for n, sty in zip ([ 2, 3 ], ['--', '-']):
        for src, col in zip(srcs, cols):
            af1 = src+'/hlv-allsky.dat'
            af2 = src+'/hlvk'+str(mpc)+'mpc-allsky.dat'
            cfn = src+'/coincs.dat'

            apl(ax, n, af1, af2, cfn, src.upper(), col, sty)
    
    loc = 'upper left' if mpc == 25 else 'lower right'

    ax.legend(fontsize=fsiz-1, loc=loc, framealpha=0.95)

    txt = 'KAGRA '+str(mpc)+' Mpc'

    fig.text(0.51, 0.96, txt, ha='center', va='top', size=fsiz+5)

    return fig

for r in [ 8, 15, 25 ]:
    fig = plot(r)
    fig.show()
    fig.savefig('acmp_O3_'+str(r)+'Mpc.pdf')
